---------------------------------------------------------------------------
--                                                                       --
--  Module      : bscntr.vhd     Last Update: 06/30/06                   --
--                                                                       --
--  Description : This is the Address counter for the BRAM scrubber      --
--                                                                       --
--                                                                       --
--  This is the counter used to cycle through the address on port B for  --
--  Scrubbing. This Part of the design must be triplicated with the      --
--  Xilinx TMR Tool.                                                     --
--                                                                       --
--  Dependancies : bscntr.vhd                                            --
--                                                                       --
--  Designer    : Greg Miller                                            --
--                                                                       --
--  Company     : Xilinx, Inc.                                           --
--                                                                       --
--  Revision    : 1.0 - Initial Release                                  --
--                                                                       --
--  Disclaimer  : THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY    --
--                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY       --
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR     --
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.         --
--                THEY ARE ONLY INTENDED TO BE USED BY XILINX            --
--                CUSTOMERS, AND WITHIN XILINX DEVICES.                  --
--                                                                       --
--                Copyright (c) 2000 Xilinx, Inc.                        --
--                All rights reserved                                    --
--                                                                       --
---------------------------------------------------------------------------

library IEEE;
use IEEE.Std_Logic_1164.all;
use IEEE.Std_logic_arith.all;
use ieee.std_logic_unsigned.all;
library work;
use work.bram_scrubber_pkg.all;

entity BSCNTR is
port (
   CLK : in std_logic;
   RST : in std_logic;
   SET : in std_logic;
   CE : in std_logic;
   ADDRESS : out std_logic_vector(addr_width-1 downto 0)
);
end BSCNTR;

Architecture rtl of BSCNTR is


Signal Count : std_logic_vector(addr_width downto 0);
Signal Count_s : std_logic_vector(addr_width downto 0);

Begin

process(CLK, SET, RST, CE) 
begin 
    if CLK'event and CLK ='1' then
          if (RST = '1') then
               Count <= (others => '0');
          elsif (SET = '1') then
               Count <= (others => '1');
          elsif (CE = '1') then
               Count <= Count_s;
          end if;
     end if;
end process;

process(Count, Count_s)
begin
     Count_s <= Count + 1;
end process;

ADDRESS <= Count(addr_width downto 1); -- one bit to hold address line -
--For faster scrubbing (will hold for two clock cycles)
--long enough to be clocked in

end rtl;
