---------------------------------------------------------------------------
--                                                                       --
--  Module      : Dut_2vP40_BRAM_SCRUBBER_STANDALONE_TOP_TEST.vhd        --     
--                Last Update: 06/30/06                                  --
--                                                                       --
--  Description : Instantiates BRAMs for the BRAM Scrubber               --
--                                                                       --
--                                                                       --
-- This is a top level wrapper for the 2VP radiation test board used by  --
-- Xilinx. This is only an example and the use will have a different     --
-- Top level wrapper if needed. This part of the design is               -- 
-- hand-triplicated and is not to be run though TMR Tool.                --
--                                                                       --
--  Dependancies : bram_scrubber_pkg.vhd                                 --
--                                                                       --
--  Designer    : Greg Miller                                            --
--                                                                       --
--  Company     : Xilinx, Inc.                                           --
--                                                                       --
--  Revision    : 1.0 - Initial Release                                  --
--                                                                       --
--  Disclaimer  : THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY    --
--                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY       --
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR     --
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.         --
--                THEY ARE ONLY INTENDED TO BE USED BY XILINX            --
--                CUSTOMERS, AND WITHIN XILINX DEVICES.                  --
--                                                                       --
--                Copyright (c) 2000 Xilinx, Inc.                        --
--                All rights reserved                                    --
--                                                                       --
---------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
Library work;
use work.bram_scrubber_pkg.all;

--NOTE: Use the Package to Choose BRAM size. Change the addr_width and
-- Data_width as needed. To get
-- your addr_width : size of log2(size of BRAM/8).
-- Example 64k bits is /8 = 8129 = 13 bits of address (used in a later
-- calulation

entity  Dut_2vp40_BRAM_SCRUBBER_STANDALONE_TOP_TEST is
     port(
     sngl_bus      : in      std_logic_vector(63 downto 0);    
     clk_tr0       : in      std_logic;     
     clk_tr1       : in      std_logic;     
     clk_tr2       : in      std_logic;  
--     ce_tr0        : in      std_logic;   
--     ce_tr1        : in      std_logic; 
--     ce_tr2        : in      std_logic; 
     tmv_bus_TR0   : out     std_logic_vector(15 downto 0); 
     tmv_bus_TR1   : out     std_logic_vector(15 downto 0); 
     tmv_bus_TR2   : out     std_logic_vector(15 downto 0)

	   ----------Optional Ports for Testing ---------
      -- CLK In from DUT--
--      local_MGT_clk_AN        : in std_logic; 
--      local_MGT_clk_AP        : in std_logic;
--      gen_A_in_clk            : out std_logic; --Clock to MB and CS is needed
--	 LVDS_OSC_EN             : out std_logic;

     ----------OPTIONAL LED Alive Indication--------------------
--	  DUT_rst_sw1_L        : in std_logic; --RST SW1 - AA34
--	  LED_OUT              : out std_logic_vector(9 downto 7);
	 ------------------------------------------------------------
	 -------OPTIONAL For Testing---------------------------------
--	 WEN_B_out_TR0           : out std_logic;    --WEN_B_out_TR0  
--	 WEN_B_out_TR1           : out std_logic;    --WEN_B_out_TR1
--	 WEN_B_out_TR2           : out std_logic;    --WEN_B_out_TR2

--      Addr_B_out_TR0          : out std_logic_vector(7 downto 0);
--      Addr_B_out_TR1          : out std_logic_vector(7 downto 0);
--      Addr_B_out_TR2          : out std_logic_vector(7 downto 0)

	 ------------------------------------------------------------
	 
);
end  Dut_2vp40_BRAM_SCRUBBER_STANDALONE_TOP_TEST;

architecture TMR of  Dut_2vp40_BRAM_SCRUBBER_STANDALONE_TOP_TEST is
 
component bram_scrubber_standalone_top_test_xtmr
  port (
     pwr_in_TR0               : in    std_logic; 
     pwr_in_TR1               : in    std_logic; 
     pwr_in_TR2               : in    std_logic;
     ren_TR0             : in    std_logic;     
     ren_TR1             : in    std_logic;     
     ren_TR2             : in    std_logic;     
     clk_TR0             : in    std_logic; -- clk0, clk1, clk2    
     clk_TR1             : in    std_logic; -- clk0, clk1, clk2    
     clk_TR2             : in    std_logic; -- clk0, clk1, clk2
     ce_tr0              : in      std_logic;  
     ce_tr1              : in      std_logic;
     ce_tr2              : in      std_logic;   

     wen_TR0             : in    std_logic_vector(0 downto 0); 
     wen_TR1             : in    std_logic_vector(0 downto 0); 
     wen_TR2             : in    std_logic_vector(0 downto 0); 
     Address_TR0         : in      std_logic_vector(15 downto 0); 
     Address_TR1         : in      std_logic_vector(15 downto 0); 
     Address_TR2         : in      std_logic_vector(15 downto 0); 
     Data_Out_TR0        : out     std_logic_vector(15 downto 0); 
     Data_Out_TR1        : out     std_logic_vector(15 downto 0); 
     Data_Out_TR2        : out     std_logic_vector(15 downto 0);

	   ----------Optional Ports for Testing ---------
      -- CLK In from DUT--
      local_MGT_clk_AN        : in    std_logic; 
      local_MGT_clk_AP        : in    std_logic; 
      gen_A_in_clk            : out    std_logic; 
	 LVDS_OSC_EN             : out    std_logic; 

     ----------OPTIONAL LED Alive Indication--------------------
      DUT_rst_sw1_L           : in    std_logic; 
	 LED_OUT                 : out    std_logic_vector(9 downto 7);
	 ------------------------------------------------------------

	 --------Optional For testing------------------------------
	 WEN_B_out_TR0           : out std_logic;
	 WEN_B_out_TR1           : out std_logic;
	 WEN_B_out_TR2           : out std_logic;

	 Addr_b_out_TR0          : out std_logic_vector(7 downto 0); --First and last nibble
	 Addr_b_out_TR1          : out std_logic_vector(7 downto 0); --First and last nibble
	 Addr_b_out_TR2          : out std_logic_vector(7 downto 0) --First and last nibble
	 
	 ----------------------------------------------------------

	 
);
end component;

attribute syn_black_box : boolean;
attribute syn_noprune : boolean;


Signal Data_Out_TR0_s, Data_Out_TR1_s, Data_Out_TR2_s : std_logic_vector(15 downto 0);


----------------------------------------------------------------------------------

--Optional for testing------------------------------------------------------------
--signal Addr_b_out_TR0_s                          : std_logic_vector(7 downto 0);
--signal Addr_b_out_TR1_s                          : std_logic_vector(7 downto 0);
--signal Addr_b_out_TR2_s                          : std_logic_vector(7 downto 0);
----------------------------------------------------------------------------------
----------------------------------------------------------------------------------

begin

-- LVDS_OSC_EN <= '1'; --Enable DUT LVDS osc

---------BRAM Scrubber Testing------------------------------------------------------
bram_scrubber_standalone_top_test_xtmr_inst : bram_scrubber_standalone_top_test_xtmr
port map (
         pwr_in_TR0          => sngl_bus(16), 
         pwr_in_TR1          => sngl_bus(33), 
         pwr_in_TR2          => sngl_bus(50),           
         ren_TR0             => sngl_bus(51), 
         ren_TR1             => sngl_bus(52), 
         ren_TR2             => sngl_bus(53), 
         clk_TR0             => clk_tr0, 
         clk_TR1             => clk_tr1, 
         clk_TR2             => clk_tr2, 
         ce_tr0              => sngl_bus(57),
         ce_tr1              => sngl_bus(58),
         ce_tr2              => sngl_bus(59),
         wen_TR0             => sngl_bus(54 downto 54), 
         wen_TR1             => sngl_bus(55 downto 55), 
         wen_TR2             => sngl_bus(56 downto 56),
         Address_TR0         => sngl_bus(15 downto 0), 
         Address_TR1         => sngl_bus(32 downto 17), 
         Address_TR2         => sngl_bus(49 downto 34), 
         Data_Out_TR0        => Data_Out_TR0_s, 
         Data_Out_TR1        => Data_Out_TR1_s, 
         Data_Out_TR2        => Data_Out_TR2_s

         ----------Optional Ports for Testing ---------
      -- CLK In from DUT--
--      local_MGT_clk_AN        => local_MGT_clk_AN,
--      local_MGT_clk_AP        => local_MGT_clk_AP,
--      gen_A_in_clk            => gen_A_in_clk,
--		LVDS_OSC_EN             => LVDS_OSC_EN,

     ----------OPTIONAL LED Alive Indication--------------------
--		DUT_rst_sw1_L           => DUT_rst_sw1_L,
--		LED_OUT                 => LED_OUT,
	 ------------------------------------------------------------
	 --------OPTIONAL For Testing--------------------------------
--	 WEN_B_out_TR0    =>       WEN_B_out_TR0,
--	 WEN_B_out_TR1    =>       WEN_B_out_TR1,
--	 WEN_B_out_TR2    =>       WEN_B_out_TR2,

--	 Addr_b_out_TR0   =>       Addr_b_out_TR0_s,
--	 Addr_b_out_TR1   =>       Addr_b_out_TR1_s,
--	 Addr_b_out_TR2   =>       Addr_b_out_TR2_s
	 
	 ------------------------------------------------------------
);

--Addr_b_out_TR0   <= Addr_b_out_TR0_s;
--Addr_b_out_TR1   <= Addr_b_out_TR1_s;
--Addr_b_out_TR2   <= Addr_b_out_TR2_s;

--tmv_bus_TR0 <= Data_Out_TR0_s;
--tmv_bus_TR1 <= Data_Out_TR1_s;
--tmv_bus_TR2 <= Data_Out_TR2_s;

end TMR;

