---------------------------------------------------------------------------
--                                                                       --
--  Module      : bram_scrubber_inst_bitw.vhd     Last Update: 06/30/06  --
--                                                                       --
--  Description : Instantiates BRAMs for the BRAM Scrubber               --
--                                                                       --
--                                                                       --
-- Instantited RAMs with separate write enables.                         --
-- Use for only for Singla Data Bits... (Change code to accommodate other--
-- sizes)                                                                --
-- For example : can use 16kx1 but not 8kx2. ().                         --
-- You can change the Package file/This file to make other BRAMs but     --
-- must match a BRAM primitive (see Data Sheet).                         --
--                                                                       --
--  Dependancies : bram_scrubber_pkg.vhd                                 --
--                                                                       --
--  Designer    : Greg Miller                                            --
--                                                                       --
--  Company     : Xilinx, Inc.                                           --
--                                                                       --
--  Revision    : 1.0 - Initial Release                                  --
--                                                                       --
--  Disclaimer  : THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY    --
--                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY       --
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR     --
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.         --
--                THEY ARE ONLY INTENDED TO BE USED BY XILINX            --
--                CUSTOMERS, AND WITHIN XILINX DEVICES.                  --
--                                                                       --
--                Copyright (c) 2000 Xilinx, Inc.                        --
--                All rights reserved                                    --
--                                                                       --
---------------------------------------------------------------------------
library IEEE; 
use IEEE.std_logic_1164.all; 
use IEEE.std_logic_unsigned.all; 
Library work;
use work.bram_scrubber_pkg.all;

entity raminfr is 
  port( 
    clka  : in  std_logic; 
    clkb  : in  std_logic; 
    ena   : in  std_logic; 
    enb   : in  std_logic; 
    wea   : in  std_logic; 
    web   : in  std_logic_vector(data_width/we_num-1 downto 0); 
    SSRA  : in std_logic;
    SSRB  : in std_logic;
    addra : in  std_logic_vector((addr_width-1) downto 0); 
    addrb : in  std_logic_vector((addr_width-1) downto 0); 
    dia   : in  std_logic_vector((data_width/we_num-1) downto 0); 
    dib   : in  std_logic_vector((data_width/we_num-1) downto 0); 
    doa   : out std_logic_vector((data_width/we_num-1) downto 0); 
    dob   : out std_logic_vector((data_width/we_num-1) downto 0)); 
end raminfr; 

architecture syn of raminfr is

-------BRAM component---------------------
------------------------------------------
component RAMB16_S1_S1                                   --This primitive can be changed
port (    CLKA      : in std_logic;
          ENA       : in std_logic;
          SSRA      : in std_logic;
          WEA       : in std_logic;
          DIA       : in std_logic_vector(0 to 0);
          ADDRA     : in std_logic_vector(addr_width-1 downto 0);
          DOA       : out std_logic_vector(0 to 0);
          CLKB      : in std_logic;
          ENB       : in std_logic;
          SSRB      : in std_logic;
          WEB       : in std_logic;
          DIB       : in std_logic_vector(0 to 0);
          ADDRB     : in std_logic_vector(addr_width-1 downto 0);
          DOB       : out std_logic_vector(0 to 0)
          );
end component;


begin 

br: FOR i in 0 to (data_width/we_num - 1) Generate
     ramb16_s1_s1_inst : RAMB16_S1_S1
     port map(      CLKA      => clka,
                    ENA       => ena,
                    SSRA      => SSRA,
                    WEA       => wea,
                    DIA       => DIA(i downto i),
                    ADDRA     => addra,
                    DOA       => DOA(i downto i),
                    CLKB      => clkb,
                    ENB       => enb,
                    SSRB      => SSRB,
                    WEB       => web(i),
                    DIB       => DIB(i downto i),
                    ADDRB     => ADDRB,
                    DOB       => DOB(i downto i)
    );
end Generate;

end syn; 

--######################################################################
--#######Generate the BRAMs based on the number of Write Enables########
--######################################################################

library IEEE; 
use IEEE.std_logic_1164.all; 
use IEEE.std_logic_unsigned.all; 
Library work;
use work.bram_scrubber_pkg.all;

entity bram_scrubber_bram_bitw is 
  port( 
    clka  : in  std_logic; 
    clkb  : in  std_logic; 
    ena   : in  std_logic; 
    enb   : in  std_logic; 
    wea   : in  std_logic_vector((we_num-1) downto 0); 
    web  : in std_logic_vector(data_width-1 downto 0); --for separate we testing
    SSRA : in std_logic;
    SSRB : in std_logic;
    addra : in  std_logic_vector((addr_width-1) downto 0); 
    addrb : in  std_logic_vector((addr_width-1) downto 0); 
    dia   : in  std_logic_vector((data_width-1) downto 0); 
    dib   : in  std_logic_vector((data_width-1) downto 0); 
    doa   : out std_logic_vector((data_width-1) downto 0); 
    dob   : out std_logic_vector((data_width-1) downto 0)); 
end bram_scrubber_bram_bitw; 

architecture struct of bram_scrubber_bram_bitw is 

component raminfr is
    port (
    clka  : in  std_logic; 
    clkb  : in  std_logic; 
    ena   : in  std_logic; 
    enb   : in  std_logic; 
    wea   : in  std_logic; 
    web   : in  std_logic_vector(data_width/we_num-1 downto 0); ----for separate we testing 
    SSRA  : in std_logic;
    SSRB  : in std_logic;
    addra : in  std_logic_vector((addr_width-1) downto 0); 
    addrb : in  std_logic_vector((addr_width-1) downto 0); 
    dia   : in  std_logic_vector((data_width/we_num-1) downto 0); 
    dib   : in  std_logic_vector((data_width/we_num-1) downto 0); 
    doa   : out std_logic_vector((data_width/we_num-1) downto 0); 
    dob   : out std_logic_vector((data_width/we_num-1) downto 0)
    );
end component;


begin 
bram : FOR i in (we_num-1) downto 0  Generate
    raminfr_inst : raminfr                                        
    port map (
     clka  => clka,
     clkb  => clkb, 
     ena   => ena, 
     enb   => enb, 
     wea   => wea(i), 
     web   => web( data_width/we_num + i*(data_width/we_num) - 1 downto data_width/we_num*(i+1) - data_width/we_num ),
     SSRA => SSRA,
     SSRB => SSRB,
     addra => addra,
     addrb => addrb,
     dia   => dia( data_width/we_num + i*(data_width/we_num) - 1 downto data_width/we_num*(i+1) - data_width/we_num ),
     dib   => dib( data_width/we_num + i*(data_width/we_num) - 1 downto data_width/we_num*(i+1) - data_width/we_num ),
     doa   => doa( data_width/we_num + i*(data_width/we_num) - 1 downto data_width/we_num*(i+1) - data_width/we_num ),
     dob   => dob( data_width/we_num + i*(data_width/we_num) - 1 downto data_width/we_num*(i+1) - data_width/we_num )
    );

End Generate;
end struct;
