---------------------------------------------------------------------------
--                                                                       --
--  Module      : bram_scrubber_data_voter.vhd     Last Update: 06/30/06 --
--                                                                       --
--  Description : This is a sub module for the bram_scrubber_wrapper_tmr.--
-- This design votes on the data from the BRAMs. Clock is inverted Here  --                                               --
--                                                                       --
--                                                                       --
--                                                                       --
--  Dependancies : bram_scrubber_pkg.vhd                                 --
--                                                                       --
--  Designer    : Greg Miller                                            --
--                                                                       --
--  Company     : Xilinx, Inc.                                           --
--                                                                       --
--  Revision    : 1.0 - Initial Release                                  --
--                                                                       --
--  Disclaimer  : THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY    --
--                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY       --
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR     --
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.         --
--                THEY ARE ONLY INTENDED TO BE USED BY XILINX            --
--                CUSTOMERS, AND WITHIN XILINX DEVICES.                  --
--                                                                       --
--                Copyright (c) 2000 Xilinx, Inc.                        --
--                All rights reserved                                    --
--                                                                       --
---------------------------------------------------------------------------

library IEEE;
use IEEE.Std_Logic_1164.all;
use IEEE.Std_logic_arith.all;
use ieee.std_logic_unsigned.all;
library work;
use work.bram_scrubber_pkg.all;

entity bram_scrubber_data_voter is
port (
     TR0     : in std_logic_vector(data_width-1 downto 0);
     TR1     : in std_logic_vector(data_width-1 downto 0); 
     TR2     : in std_logic_vector(data_width-1 downto 0);
     Din_B   : out std_logic_vector(data_width-1 downto 0);
     ERROR_B : out std_logic_vector(data_width-1 downto 0)
     );
end bram_scrubber_data_voter;

Architecture rtl of bram_scrubber_data_voter is

component TRV_LUT
port (
   TR0 : in std_logic;
   TR1 : in std_logic;
   TR2 : in std_logic;
   V   : out std_logic);
end component;

component TMR_DET
port (
   Prime : in std_logic;
   redA  : in std_logic;
   redB  : in std_logic;
   ERROR : out std_logic);
end component;

Begin

------------------------------------------

data_voter : FOR i in 0 to (data_width-1) Generate
DATA_B_Voter : TRV_LUT
   port map (
             TR0 => TR0(i),
             TR1 => TR1(i), 
             TR2 => TR2(i), 
             V => Din_B(i)
            );

DATA_B_ErrDet : TMR_DET
   port map (
             Prime => TR0(i),
             redA  => TR1(i),
             redB  => TR2(i), 
             ERROR => ERROR_B(i)
            );

End Generate;
end rtl;

------------------------------------------
library IEEE;
use IEEE.Std_Logic_1164.all;
use IEEE.Std_logic_arith.all;

entity TMR_DET is
port (
   Prime : in std_logic;
   redA : in std_logic;
   redB : in std_logic;
   ERROR : out std_logic);
end TMR_DET;

Architecture rtl of TMR_DET is

Begin
	ERROR <= (Prime xor redA) and (Prime xor redB);
end rtl;

-----------------------------------------------------------------
library IEEE;
use IEEE.Std_Logic_1164.all;
use IEEE.Std_logic_arith.all;

entity TRV_LUT is
port (
   TR0 : in std_logic;
   TR1 : in std_logic;
   TR2 : in std_logic;
   V : out std_logic);
end TRV_LUT;

Architecture rtl of TRV_LUT is

Begin
	V <= (TR0 and TR1) or (TR1 and TR2) or (TR2 and TR0);
end rtl;


