---------------------------------------------------------------------------
--                                                                       --
--  Module      : bram_scrubber_pkg.vhd           Last Update: 06/30/06  --
--                                                                       --
--  Description : VHDL Package needed for All BRAM Scrubber Source       --
--                                                                       --
--  Change the Data/Address size or BE size in this file                 --
--                                                                       --
--                                                                       --
--  Dependancies :                                                       --
--                                                                       --
--  Designer    : Greg Miller                                            --
--                                                                       --
--  Company     : Xilinx, Inc.                                           --
--                                                                       --
--  Revision    : 1.0 - Initial Release                                  --
--                                                                       --
--  Disclaimer  : THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY    --
--                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY       --
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR     --
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.         --
--                THEY ARE ONLY INTENDED TO BE USED BY XILINX            --
--                CUSTOMERS, AND WITHIN XILINX DEVICES.                  --
--                                                                       --
--                Copyright (c) 2000 Xilinx, Inc.                        --
--                All rights reserved                                    --
--                                                                       --
---------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

package bram_scrubber_pkg is

-- Declare constants
-- This is where the user can change the size of the BRAMs
    constant data_width : integer := 64; 
    constant addr_width : integer := 14;
    constant we_num     : integer := 1;

--Components
-- Port B address Counter Component
component BSCNTR_XTMR
port (
     internal_gnd_tr0 : in std_logic;
     internal_gnd_tr1 : in std_logic;
     internal_gnd_tr2 : in std_logic;
     internal_vcc_tr0 : in std_logic;
     internal_vcc_tr1 : in std_logic;
     internal_vcc_tr2 : in std_logic;
     CLK_TR0 : in std_logic;
     CLK_TR1 : in std_logic;
     CLK_TR2 : in std_logic;
     SET_TR0 : in std_logic;
     SET_TR1 : in std_logic;
     SET_TR2 : in std_logic;
     RST_TR0 : in std_logic;
     RST_TR1 : in std_logic;
     RST_TR2 : in std_logic;
     CE_TR0 : in std_logic;
     CE_TR1 : in std_logic;
     CE_TR2 : in std_logic;
     ADDRESS_TR0 : out std_logic_vector((addr_width-1) downto 0);
     ADDRESS_TR1 : out std_logic_vector((addr_width-1) downto 0);
     ADDRESS_TR2 : out std_logic_vector((addr_width-1) downto 0)
     );
end component;

--Address Comparator Component
component Comparator_TMR
port (

     CEN_TR0 : out std_logic;
     CEN_TR1 : out std_logic;
     CEN_TR2 : out std_logic;

     ENB_TR0 : out std_logic;
     ENB_TR1 : out std_logic;
     ENB_TR2 : out std_logic;

     ADDR_A_TR0 : in std_logic_vector ((addr_width-1) downto 0);
     ADDR_A_TR1 : in std_logic_vector ((addr_width-1) downto 0);
     ADDR_A_TR2 : in std_logic_vector ((addr_width-1) downto 0);
     ADDR_B_TR0 : in std_logic_vector ((addr_width-1) downto 0);
     ADDR_B_TR1 : in std_logic_vector ((addr_width-1) downto 0);
     ADDR_B_TR2 : in std_logic_vector ((addr_width-1) downto 0);

     WEN_A_TR0  : in std_logic_vector((we_num-1)     downto 0);
     WEN_A_TR1  : in std_logic_vector((we_num-1)     downto 0);
     WEN_A_TR2  : in std_logic_vector((we_num-1)     downto 0);

     WEN_B_TR0 : out std_logic_vector(data_width-1 downto 0);
     WEN_B_TR1 : out std_logic_vector(data_width-1 downto 0);
     WEN_B_TR2 : out std_logic_vector(data_width-1 downto 0);

     ERR_B_TR0 : in std_logic_vector (data_width-1 downto 0);
     ERR_B_TR1 : in std_logic_vector (data_width-1 downto 0);
     ERR_B_TR2 : in std_logic_vector (data_width-1 downto 0)
          );
end component;

component bram_scrubber_bram
     port (
          CLKA  : in std_logic;
          ENA   : in std_logic;
          SSRA  : in std_logic;   --Add For Full HL Extraction
          WEA   : in std_logic_vector((we_num-1) downto 0);
          DIA   : in std_logic_vector(data_width-1 downto 0);
          ADDRA : in std_logic_vector(addr_width-1 downto 0);
          DOA   : out std_logic_vector(data_width-1 downto 0);
          CLKB  : in std_logic;
          ENB   : in std_logic;
          SSRB  : in std_logic;   --Add For Full HL Extraction
          WEB   : in std_logic;
          DIB   : in std_logic_vector(data_width-1 downto 0);
          ADDRB : in std_logic_vector(addr_width-1 downto 0);
          DOB   : out std_logic_vector(data_width-1 downto 0)
            );
end component;

component bram_scrubber_bram_bitw
     port (
          CLKA  : in std_logic;
          ENA   : in std_logic;
          SSRA  : in std_logic;   --Add For Full HL Extraction
          WEA   : in std_logic_vector((we_num-1) downto 0);
          DIA   : in std_logic_vector(data_width-1 downto 0);
          ADDRA : in std_logic_vector(addr_width-1 downto 0);
          DOA   : out std_logic_vector(data_width-1 downto 0);
          CLKB  : in std_logic;
          ENB   : in std_logic;
          SSRB  : in std_logic;   --Add For Full HL Extraction
          WEB   : in std_logic_vector(data_width-1 downto 0);
          DIB   : in std_logic_vector(data_width-1 downto 0);
          ADDRB : in std_logic_vector(addr_width-1 downto 0);
          DOB   : out std_logic_vector(data_width-1 downto 0)
            );
end component;

component bram_scrubber_data_voter
     port (
          TR0                 : in std_logic_vector(data_width-1 downto 0);
          TR1                 : in std_logic_vector(data_width-1 downto 0); 
          TR2                 : in std_logic_vector(data_width-1 downto 0);
          Din_B               : out std_logic_vector(data_width-1 downto 0);
          ERROR_B             : out std_logic_vector(data_width-1 downto 0)
        );
end component;

component TRV_LUT
port (
   TR0 : in std_logic;
   TR1 : in std_logic;
   TR2 : in std_logic;
   V : out std_logic);
end component;

component TMR_DET
port (
   Prime : in std_logic;
   redA : in std_logic;
   redB : in std_logic;
   ERROR : out std_logic);
end component;

component CEN_bscntr_tmr 
port (
     BOGUS_VCC : in std_logic;
     CEN_TR0 : in std_logic; 
     CEN_TR1 : in std_logic;
     CEN_TR2 : in std_logic; 
     CEN_bscntr_tmr : out std_logic);
end component;

component CENVTMR 
port (
     BOGUS_GND : in std_logic;
     CENV_TR0 : in std_logic; 
     CENV_TR1 : in std_logic;
     CENV_TR2 : in std_logic; 
     CENVTMR  : out std_logic);
end component; 

end bram_scrubber_pkg;
