---------------------------------------------------------------------------
--                                                                       --
--  Module      : bram_scrubber_standalone_top_test.vhd                  --
--                Last Update: 06/30/06                                  --
--                                                                       --
--  Description : Top Level Instantiation of the BRAM scrubber test      --
--                                                                       --
--                                                                       --
-- This is a standalone test module that instantiates 3 BRAMs and is to be 
-- used in the TMR tool to replace the BRAMs (1 of them) - This is       --
-- based on the bram scrubber package and is adjustable.                 --
-- use all BRAMs in a 2VP40(192). Mux the Output Data.                   --
--                                                                       --
--  Dependancies : bram_scrubber_standalone_top_test.vhd                 --
--                                                                       --
--  Designer    : Greg Miller                                            --
--                                                                       --
--  Company     : Xilinx, Inc.                                           --
--                                                                       --
--  Revision    : 1.0 - Initial Release                                  --
--                                                                       --
--  Disclaimer  : THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY    --
--                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY       --
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR     --
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.         --
--                THEY ARE ONLY INTENDED TO BE USED BY XILINX            --
--                CUSTOMERS, AND WITHIN XILINX DEVICES.                  --
--                                                                       --
--                Copyright (c) 2000 Xilinx, Inc.                        --
--                All rights reserved                                    --
--                                                                       --
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
Library work;
use work.bram_scrubber_pkg.all;

entity BRAM_SCRUBBER_STANDALONE_TOP_TEST is
     port(
      ren            : in    std_logic;     
      clk            : in    std_logic;  
      wen            : in    std_logic_vector(we_num-1 downto 0); 
      Address        : in    std_logic_vector((addr_width-1)+2 downto 0);  
      ce             : in    std_logic;
      Data_Out       : out   std_logic_vector(data_width/4-1 downto 0)

      ----------Optional Ports for Testing ---------
      -- CLK In from DUT--
--     local_MGT_clk_AN        : in std_logic; 
--      local_MGT_clk_AP        : in std_logic;
--      gen_A_in_clk            : out std_logic; --Clock to MB and CS is needed
--      LVDS_OSC_EN             : out std_logic;

     ----------OPTIONAL LED Alive Indication--------------------
--       DUT_rst_sw1_L        : in std_logic; --RST SW1 - AA34
--       LED_OUT              : out std_logic_vector(9 downto 7);
     ------------------------------------------------------------
    ----------------Optional For testing------------------------
--       WEN_B_out            : out std_logic;
--       Addr_B_out           : out std_logic_vector(7 downto 0)
    ------------------------------------------------------------
         
     );
end BRAM_SCRUBBER_STANDALONE_TOP_TEST;

architecture BEHAVIORAL of BRAM_SCRUBBER_STANDALONE_TOP_TEST is

-----------------------------------------------------------
-- COMPONENTS  --------------------------------------------
-----------------------------------------------------------

component bram
port (    
          internal_vcc : in std_logic;
          internal_gnd : in std_logic;
          WE   : in std_logic_vector(we_num-1 downto 0);
          EN   : in std_logic;
          SSR  : in std_logic;
          CLK  : in std_logic;
          ADDR : in std_logic_vector(addr_width-1 downto 0);
          DI   : in std_logic_vector(data_width-1 downto 0);
          DO   : out std_logic_vector(data_width-1 downto 0)
          -----------Optional------------
--          cs_clk : in std_logic;
--          WEN_B_out : out std_logic;
--          Addr_B_out : out std_logic_vector(7 downto 0)
          -------------------------------
     );
end component;


----------------------------------------------------------
--------Optional Components-------------------------------
----------------------------------------------------------
COMPONENT FDDRRSE
  PORT(
    Q    : OUT std_logic;
    C0   : IN  std_logic;
    C1   : IN  std_logic;
    CE   : IN  std_logic;
    D0   : IN  std_logic;
    D1   : IN  std_logic;
    R    : IN  std_logic;
    S    : IN  std_logic
);
END COMPONENT;

component IBUFGDS_LVDS_25
port (
     I     : in std_logic;
     IB    : in std_logic;
     O     : out std_logic
     );
end component;

component BUFG

port (    I : in std_logic;
        O : out std_logic);
end component;

component BUFGCE
port     (I  : in std_logic;
          CE : in std_logic;
          O  : out std_logic
          );
end component;

component FDRSE
    port
    (
        Q : out std_ulogic;
        C : in std_ulogic;
        CE : in std_ulogic;
        D : in std_ulogic;
        R : in std_ulogic;
        S : in std_ulogic
    );
end component;

-----------------------------------------------------------
-- USER SIGNALS -------------------------------------------
-----------------------------------------------------------
attribute syn_black_box : boolean;
attribute syn_noprune : boolean;

signal addr_int : std_logic_vector(addr_width-1 downto 0);
signal di_int : std_logic_vector(data_width-1 downto 0);

signal internal_gnd : std_logic;
signal internal_vcc : std_logic;
signal clk_s        : std_logic;
signal Data_Out_s     : std_logic_vector(data_width/4-1 downto 0);
signal Data_Out_mux_s : std_logic_vector(data_width-1 downto 0);

--------------OPTIONAL--------------------------------
--signal local_MGT_clk_BG, local_MGT_clk : std_logic;
--signal led_counter_s : std_logic_vector(28 downto 0);
--signal led_count_s   : std_logic_vector(2 downto 0);
--signal WEN_B_out_s  : std_logic;
--signal Addr_B_out_s : std_logic_vector(7 downto 0);
------------------------------------------------------

begin

internal_gnd <= '0';
internal_vcc <= '1';

----------Optional Clock for CS-----------
------------------------------------------
--clk_cs_out     <=  clk_cs_in;

addr_int <= Address(addr_width-1 downto 0);

--Just write the same data to the entire 16x4 structure.
di_int <= (Address(1) & Address(0) & Address(data_width/4-3 downto 0)) & (Address(1) & Address(0) & Address(data_width/4-3 downto 0)) & (Address(1) & Address(0) & Address(data_width/4-3 downto 0)) & (Address(1) & Address(0) & Address(data_width/4-3 downto 0));

------------------------------------------
BUFG_MAIN_CLK : BUFGCE
         port map
         (
             I => clk,
             CE => ce,
             O => clk_s
         );
------------------------------------------

-----------------------------------------------------------
--  BRAM INSTANTIATIONS -----------------------------------
-----------------------------------------------------------

bram_0 : bram
port map (
               internal_vcc => internal_vcc, 
               internal_gnd => internal_gnd, 
               WE => wen, 
               EN => ren, 
               SSR => internal_gnd, 
               CLK => clk_s,  
               ADDR => addr_int,
               DI   => di_int, 
               DO   => Data_Out_mux_s
            ------Optional--------------
--              cs_clk => local_MGT_clk, --Over sample for Chipscope
--              WEN_B_out => WEN_B_out_s,   --For Observing Scrubbing Engine
--              Addr_B_out => Addr_B_out_s  --Observe the Address_B
            ----------------------------
            );

--Optional For Testing----  
--addr_b_ff : FOR i in 7 downto 0  Generate

 
-- addr_b_inst : FDRSE   
--    port map (
--     Q  => Addr_B_out(i),
--     C  => clk_s,
--     CE => internal_vcc,
--     D  => Addr_B_out_s(i),
--     R  => internal_gnd,
--     S  => internal_gnd
--);       
--End Generate;

---Now, Mux the data output into 4 chuncks of 16-bits---------
---Based on the upper two address bits------------------------
MUX_DATA : process (Address(15 downto 14), Data_Out_mux_s)
begin

     case (Address(15 downto 14)) is
          when "00" =>
               Data_Out_s <= Data_Out_mux_s(15 downto 0);
          when "01" =>
               Data_Out_s <= Data_Out_mux_s(31 downto 16);
          when "10" =>
               Data_Out_s <= Data_Out_mux_s(47 downto 32);
          when "11" =>
               Data_Out_s <= Data_Out_mux_s(63 downto 48);
          when others =>
               Data_Out_s <= Data_Out_mux_s(15 downto 0);
     end case;
     
end process MUX_DATA;

data_out_ff0 : FOR i in (data_width/4-1) downto 0  Generate
data_out_inst0 : FDRSE   
    port map (
     Q  => Data_Out(i),
     C  => clk_s,
     CE => internal_vcc,
     D  => Data_Out_s(i),
     R  => internal_gnd,
     S  => internal_gnd
);    
End Generate;

--Optional For testing--
--WEN_B_ff : FDRSE
--port map (
--     Q =>  WEN_B_out,
--     C =>  clk_s,
--     CE => internal_vcc,
--     D  => WEN_B_out_s,
--     R  => internal_gnd,
--     S  => internal_gnd   
--);

-----------------OPTIONAL----------------------------------

--LVDS_OSC_EN <= '1'; --Enable DUT LVDS osc

--pix0clk: FDDRRSE
--PORT MAP(
--      Q    => gen_A_in_clk,
--    C0   => local_MGT_clk,
--    C1   => NOT local_MGT_clk,
--    CE   => '1',
--    D0   => '1',
--    D1   => '0',
--    R    => '0',
--    S    => '0'   
--    );

--IBUFGDS_SYS_TOP : IBUFGDS_LVDS_25
--         port map (
--             I  => local_MGT_clk_AP,
--             IB => local_MGT_clk_AN,
--             O  => local_MGT_clk_BG
--             );

--BUFG_REF_TOP : BUFG
--         port map
--         (
--             I => local_MGT_clk_BG,
--             O => local_MGT_clk
--         );

--COUNT : process (local_MGT_clk , DUT_rst_sw1_L)
--begin
--     if (DUT_rst_sw1_L = '0') then
--        led_counter_s <= (others => '0');
--        led_count_s <= (others => '0');
--     elsif rising_edge(local_MGT_clk) then
--          if (led_counter_s(led_counter_s'high) = '1') then
--                led_count_s <= unsigned(led_count_s) + '1';
--          end if;
--          led_counter_s <= unsigned(led_counter_s) + '1';
--       end if;
--end process COUNT;

--LED_OUT(9 downto 7) <= led_count_s;

end BEHAVIORAL;
