---------------------------------------------------------------------------
--                                                                       --
--  Module      : bram_scrubber_wrapper_bitw.vhd  Last Update: 06/30/06  --
--                                                                       --
--  Description : This is the actual BRAM scrubber engine                --
--                                                                       --
--                                                                       --
-- This is to be used to replace BRAMS in a design that will require     --
-- BRAM scrubbing as a macro in the TMR Tool. This part of the design    --
-- was TMR'd by HAND.                                                    --
--                                                                       --
--                                                                       --
--  Dependancies : bram_scrubber_pkg.vhd                                 --
--                                                                       --
--  Designer    : Greg Miller                                            --
--                                                                       --
--  Company     : Xilinx, Inc.                                           --
--                                                                       --
--  Revision    : 1.0 - Initial Release                                  --
--                                                                       --
--  Disclaimer  : THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY    --
--                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY       --
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR     --
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.         --
--                THEY ARE ONLY INTENDED TO BE USED BY XILINX            --
--                CUSTOMERS, AND WITHIN XILINX DEVICES.                  --
--                                                                       --
--                Copyright (c) 2000 Xilinx, Inc.                        --
--                All rights reserved                                    --
--                                                                       --
---------------------------------------------------------------------------
library IEEE;
use IEEE.Std_Logic_1164.all;
use IEEE.Std_logic_arith.all;

entity CEN_bscntr_tmr is
port (
     BOGUS_VCC : in std_logic;
     CEN_TR0 : in std_logic; 
     CEN_TR1 : in std_logic;
     CEN_TR2 : in std_logic; 
     CEN_bscntr_tmr  : out std_logic);
end CEN_bscntr_tmr;

Architecture rtl of CEN_bscntr_tmr is

Begin
     CEN_bscntr_tmr <= CEN_TR0 and CEN_TR1 and CEN_TR2 and BOGUS_VCC;
end rtl;
----------------------------------------

----------------------------------------
library IEEE;
use IEEE.Std_Logic_1164.all;
use IEEE.Std_logic_arith.all;

entity CENVTMR is
port (
     BOGUS_GND : in std_logic;
     CENV_TR0 : in std_logic; 
     CENV_TR1 : in std_logic;
     CENV_TR2 : in std_logic; 
     CENVTMR  : out std_logic);
end CENVTMR;

Architecture rtl of CENVTMR is

Begin
     CENVTMR <= CENV_TR0 or CENV_TR1 or CENV_TR2 or BOGUS_GND;
end rtl;
---------------------------------------

---------------------------------------
library IEEE;
use IEEE.Std_Logic_1164.all;
use IEEE.Std_logic_arith.all;
Library work;
use work.bram_scrubber_pkg.all;

entity Comparator_TMR is
port (
     ADDR_A_TR0       : in std_logic_vector((addr_width-1) downto 0);
     ADDR_A_TR1       : in std_logic_vector((addr_width-1) downto 0);
     ADDR_A_TR2       : in std_logic_vector((addr_width-1) downto 0);
     WEN_A_TR0        : in std_logic_vector((we_num-1)     downto 0);
     WEN_A_TR1        : in std_logic_vector((we_num-1)     downto 0);
     WEN_A_TR2        : in std_logic_vector((we_num-1)     downto 0);
     ADDR_B_TR0       : in std_logic_vector((addr_width-1) downto 0);
     ADDR_B_TR1       : in std_logic_vector((addr_width-1) downto 0);
     ADDR_B_TR2       : in std_logic_vector((addr_width-1) downto 0); 
     ERR_B_TR0        : in std_logic_vector(data_width-1 downto 0);
     ERR_B_TR1        : in std_logic_vector(data_width-1 downto 0);
     ERR_B_TR2        : in std_logic_vector(data_width-1 downto 0);
     CEN_TR0          : out std_logic;
     CEN_TR1          : out std_logic;
     CEN_TR2          : out std_logic;
     WEN_B_TR0        : out std_logic_vector(data_width-1 downto 0);
     WEN_B_TR1        : out std_logic_vector(data_width-1 downto 0);
     WEN_B_TR2        : out std_logic_vector(data_width-1 downto 0);
     ENB_TR0          : out std_logic;
     ENB_TR1          : out std_logic;
     ENB_TR2          : out std_logic
          );
end Comparator_TMR;

Architecture rtl of Comparator_TMR is

----Signals---
signal address_conflict_TR0_s, address_conflict_TR1_s, address_conflict_TR2_s : std_logic;
signal WEN_B_TR0_s, WEN_B_TR1_s, WEN_B_TR2_s                                  : std_logic_vector(data_width-1 downto 0);
--------------
begin
-----------------------------------------------
--Check address conflict--
ac0 : process(Addr_A_TR0, Addr_B_TR0, WEN_A_TR0)
begin
     if ( (Addr_A_TR0 = Addr_B_TR0) )  and (WEN_A_TR0 /= (WEN_A_TR0'range => '0'))  then
               address_conflict_TR0_s <= '1';
          else
               address_conflict_TR0_s <= '0';
     end if;
end process;

--Check address conflict--
ac1 : process(Addr_A_TR1, Addr_B_TR1, WEN_A_TR1)
begin
     if ( (Addr_A_TR1 = Addr_B_TR1) )  and (WEN_A_TR1 /= (WEN_A_TR1'range => '0'))  then
               address_conflict_TR1_s <= '1';
          else
               address_conflict_TR1_s <= '0';
     end if;
end process;

--Check address conflict--
ac2 : process(Addr_A_TR2, Addr_B_TR2, WEN_A_TR2)
begin
     if ( (Addr_A_TR2 = Addr_B_TR2) )  and (WEN_A_TR2 /= (WEN_A_TR2'range => '0'))  then
               address_conflict_TR2_s <= '1';
          else
               address_conflict_TR2_s <= '0';
     end if;
end process;
--------------------------------------------------

--Feed though WEB from errors no matter what---
WEN_B_TR0_s <= ERR_B_TR0;
WEN_B_TR1_s <= ERR_B_TR1;
WEN_B_TR2_s <= ERR_B_TR2;

WEN_B_TR0 <= WEN_B_TR0_s;
WEN_B_TR1 <= WEN_B_TR1_s;
WEN_B_TR2 <= WEN_B_TR2_s;

--------------------------------------------------
comp_tr0 : process(address_conflict_TR0_s, WEN_B_TR0_s)
begin 
    if address_conflict_TR0_s = '1' then
        ENB_TR0 <= '0';                                    --Don't Correct by turning off BRAM
        CEN_TR0 <= '1';                                    --Turn On Address Counter
    else
        ENB_TR0 <= '1'; 
        if (WEN_B_TR0_s = (WEN_B_TR0_s'range => '0')) then
            CEN_TR0 <= '1';                                --When there are no errors (WEB) then counter is on
        else
            CEN_TR0 <= '0';                                --Turn off counter so that it has time to write
        end if;
    end if;
end process;

comp_tr1 : process(address_conflict_TR1_s, WEN_B_TR1_s)
begin 
    if address_conflict_TR1_s = '1' then
        ENB_TR1 <= '0';                                    --Don't Correct by turning off BRAM
        CEN_TR1 <= '1';                                    --Turn On Address Counter
    else
        ENB_TR1 <= '1'; 
        if (WEN_B_TR1_s = (WEN_B_TR1_s'range => '0')) then
            CEN_TR1 <= '1';                                --When there are no errors (WEB) then counter is on
        else
            CEN_TR1 <= '0';                                --Turn off counter so that it has time to write
        end if;
    end if;
end process;

comp_tr2 : process(address_conflict_TR2_s, WEN_B_TR2_s)
begin 
    if address_conflict_TR2_s = '1' then
        ENB_TR2 <= '0';                                    --Don't Correct by turning off BRAM
        CEN_TR2 <= '1';                                    --Turn On Address Counter
    else
        ENB_TR2 <= '1'; 
        if (WEN_B_TR2_s = (WEN_B_TR2_s'range => '0')) then
            CEN_TR2 <= '1';                                --When there are no errors (WEB) then counter is on
        else
            CEN_TR2 <= '0';                                --Turn off counter so that it has time to write
        end if;
    end if;
end process;
--------------------------------------------------
WEN_B_TR0 <= WEN_B_TR0_s;
WEN_B_TR1 <= WEN_B_TR1_s;
WEN_B_TR2 <= WEN_B_TR2_s;

end rtl;
----------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
Library work;
use work.bram_scrubber_pkg.all;

entity BRAM_SCRUBBER_WRAPPER_TMR is
  port (
     internal_gnd_TR0 : in std_logic;
     internal_gnd_TR1 : in std_logic;
     internal_gnd_TR2 : in std_logic;
     internal_vcc_TR0 : in std_logic;
     internal_vcc_TR1 : in std_logic;
     internal_vcc_TR2 : in std_logic;
     WE_TR0 : in std_logic_vector((we_num-1) downto 0);
     WE_TR1 : in std_logic_vector((we_num-1) downto 0);
     WE_TR2 : in std_logic_vector((we_num-1) downto 0);
     EN_TR0 : in std_logic;
     EN_TR1 : in std_logic;
     EN_TR2 : in std_logic;
     SSR_TR0 : in std_logic;
     SSR_TR1 : in std_logic;
     SSR_TR2 : in std_logic;
     CLK_TR0 : in std_logic;
     CLK_TR1 : in std_logic;
     CLK_TR2 : in std_logic;
     ADDR_TR0 : in std_logic_vector((addr_width-1) downto 0);
     ADDR_TR1 : in std_logic_vector((addr_width-1) downto 0);
     ADDR_TR2 : in std_logic_vector((addr_width-1) downto 0);
     DI_TR0 : in std_logic_vector((data_width-1) downto 0);
     DI_TR1 : in std_logic_vector((data_width-1) downto 0);
     DI_TR2 : in std_logic_vector((data_width-1) downto 0);
     DO_TR0 : out std_logic_vector((data_width-1) downto 0);
     DO_TR1 : out std_logic_vector((data_width-1) downto 0);
     DO_TR2 : out std_logic_vector((data_width-1) downto 0)

     ------------Optional Signlas---------------------------
     -----------For testing only----------------------------
     -------------------------------------------------------
--     WEN_B_out_TR0 : out std_logic;
--     WEN_B_out_TR1 : out std_logic;
--    WEN_B_out_TR2 : out std_logic;

--    Addr_B_out_TR0 : out std_logic_vector(7 downto 0);
--    Addr_B_out_TR1 : out std_logic_vector(7 downto 0);
--    Addr_B_out_TR2 : out std_logic_vector(7 downto 0);
    --------------------------------------------------------

     ----Optional for Fast DCM Clock------------------------
--     cs_clk_TR0    : in  std_logic;
--     cs_clk_TR1    : in  std_logic;
--     cs_clk_TR2    : in  std_logic
     -------------------------------------------------------
     
----------------------------------------------------
     --View Signals For Simulation
----------------------------------------------------

     --CENVTMR_TR0 : inout std_logic;
     --CENVTMR_TR1 : inout std_logic;
     --CENVTMR_TR2 : inout std_logic;
--     CENV_TR0 : inout std_logic;
--     CENV_TR1 : inout std_logic;
--     CENV_TR2 : inout std_logic;

--     Addr_B_TR0 : inout std_logic_vector(13 downto 0);
--     Addr_B_TR1 : inout std_logic_vector(13 downto 0);
--     Addr_B_TR2 : inout std_logic_vector(13 downto 0);

--     Din_B_TR0 : inout std_logic_vector(0 to 15);
--     Din_B_TR1 : inout std_logic_vector(0 to 15);
--     Din_B_TR2 : inout std_logic_vector(0 to 15);
          
--     Dout_B_TR0 : inout std_logic_vector(0 to 15);
--     Dout_B_TR1 : inout std_logic_vector(0 to 15);
--     Dout_B_TR2 : inout std_logic_vector(0 to 15);
--     ERR_B_TR0 : inout std_logic_vector(15 downto 0);
--     ERR_B_TR1 : inout std_logic_vector(15 downto 0);
--     ERR_B_TR2 : inout std_logic_vector(15 downto 0);
--     WEN_B_TR0 : inout std_logic_vector(15 downto 0);
--     WEN_B_TR1 : inout std_logic_vector(15 downto 0);
--     WEN_B_TR2 : inout std_logic_vector(15 downto 0);
     --CEN_bscntr_tmr_TR0: inout std_logic;
     --CEN_bscntr_tmr_TR1: inout std_logic;       
     --CEN_bscntr_tmr_TR2: inout std_logic;       
--     CEN_TR0: inout std_logic;
--     CEN_TR1: inout std_logic;     
--     CEN_TR2: inout std_logic      

  );

end BRAM_SCRUBBER_WRAPPER_TMR;

architecture rtl of BRAM_SCRUBBER_WRAPPER_TMR is

--Remove for TB
 Signal    CEN_TR0, CEN_TR1, CEN_TR2                                 : std_logic;
 Signal    ERR_B_TR0, ERR_B_TR1, ERR_B_TR2                           : std_logic_vector(data_width-1 downto 0);
 Signal    WEN_B_TR0, WEN_B_TR1, WEN_B_TR2                           : std_logic_vector(data_width-1 downto 0);
 Signal    Addr_B_TR0, Addr_B_TR1, Addr_B_TR2                        : std_logic_vector(addr_width-1 downto 0);
 Signal    Din_B_TR0, Din_B_TR1, Din_B_TR2                           : std_logic_vector(data_width-1 downto 0);
 Signal    Dout_B_TR0, Dout_B_TR1, Dout_B_TR2                        : std_logic_vector(data_width-1 downto 0);
 Signal    DO_TR0_s                                                  : std_logic_vector((data_width-1) downto 0);
 Signal    DO_TR1_s                                                  : std_logic_vector((data_width-1) downto 0);
 Signal    DO_TR2_s                                                  : std_logic_vector((data_width-1) downto 0);
 Signal    ENB_TR0_s, ENB_TR1_s, ENB_TR2_s                           : std_logic;

Signal    CLKB_TR0, CLKB_TR1, CLKB_TR2                                : std_logic;
Signal    CEN_bscntr_tmr_TR0, CEN_bscntr_tmr_TR1, CEN_bscntr_tmr_TR2  : std_logic;

-----------------------------------------------------------------------------------
----These were "internal Signals coming in" now they do not exist. Force with -----
----Signals For now. --------------------------------------------------------------
----This is because the TMR Tool Does not see these as registers-------------------
----Because this sction only sees BRAMS--------------------------------------------
Signal    internal_gnd_TR0_s : std_logic;
Signal    internal_gnd_TR1_s : std_logic;
Signal    internal_gnd_TR2_s : std_logic;
Signal    internal_vcc_TR0_s : std_logic;
Signal    internal_vcc_TR1_s : std_logic;
Signal    internal_vcc_TR2_s : std_logic;
-----------------------------------------------------------------------------------

---------------Optional------------------------------------------------------------
signal    Addr_B_out_TR0_s : std_logic_vector(7 downto 0);
signal  Addr_B_out_TR1_s : std_logic_vector(7 downto 0);
signal  Addr_B_out_TR2_s : std_logic_vector(7 downto 0);
signal  WEN_B_out_TR0_s, WEN_B_out_TR1_s, WEN_B_out_TR2_s : std_logic;
----------------------------------------------------------------------------------- 

attribute keep: string;

--attribute keep of CEN_bscntr_tmr_TR0: signal is "true";
--attribute keep of CEN_bscntr_tmr_TR1: signal is "true";
--attribute keep of CEN_bscntr_tmr_TR2: signal is "true";

--attribute keep of CENVTMR_TR0: signal is "true";
--attribute keep of CENVTMR_TR1: signal is "true";
--attribute keep of CENVTMR_TR2: signal is "true";

--attribute keep of Din_B_TR0: signal is "true";
--attribute keep of Din_B_TR1: signal is "true";
--attribute keep of Din_B_TR2: signal is "true";

-- attribute keep of Din_B_TR0_sig: signal is "true";
-- attribute keep of Din_B_TR1_sig: signal is "true";
-- attribute keep of Din_B_TR2_sig: signal is "true";

--attribute keep of Dout_B_TR0: signal is "true";
--attribute keep of Dout_B_TR1: signal is "true";
--attribute keep of Dout_B_TR2: signal is "true";

-- attribute keep of WEN_B_TR0: signal is "true";
-- attribute keep of WEN_B_TR1: signal is "true";
-- attribute keep of WEN_B_TR2: signal is "true";

--attribute keep of CEN_TR0: signal is "true";
--attribute keep of CEN_TR1: signal is "true";
--attribute keep of CEN_TR2: signal is "true";

--attribute keep of CENV_TR0: signal is "true";
--attribute keep of CENV_TR1: signal is "true";
--attribute keep of CENV_TR2: signal is "true";

--attribute keep of CLK_TR0: signal is "true";
--attribute keep of CLK_TR1: signal is "true";
--attribute keep of CLK_TR2: signal is "true";

-- attribute keep of ERR_B_TR0_sig: signal is "true";
-- attribute keep of ERR_B_TR1_sig: signal is "true";
-- attribute keep of ERR_B_TR2_sig: signal is "true";

---------------------------CS Optional-----------------------------
-------------------------------------------------------------------
 --
 --  ICON Pro core component declaration
 --
 -------------------------------------------------------------------
-- component icon
--   port
--   (
--     control0    :   out std_logic_vector(35 downto 0)
--   );
-- end component;


 -------------------------------------------------------------------
 --
 --  ILA Pro core component declaration
 --
 -------------------------------------------------------------------
-- component ila
--   port
--   (
--     control     : in    std_logic_vector(35 downto 0);
--     clk         : in    std_logic;
--     data        : in    std_logic_vector(255 downto 0);
--     trig0       : in    std_logic_vector(255 downto 0)
--   );
-- end component;

-- attribute syn_black_box : boolean;
-- attribute syn_black_box of icon : component is TRUE;
-- attribute syn_black_box of ila : component is TRUE;
-- attribute syn_noprune : boolean;
-- attribute syn_noprune of icon : component is TRUE;
-- attribute syn_noprune of ila : component is TRUE;

 -------------------------------------------------------------------
 --
 --  ILA Pro core signal declarations
 --
 -------------------------------------------------------------------
-- signal control    : std_logic_vector(35 downto 0);
-- signal trig0      : std_logic_vector(255  downto 0);

 ------------End CS Optional----------------------------------------


begin

------------------------------------------------------
-- CLock were inverted here (now in Module)
------------------------------------------------------
CLKB_TR0  <= (Clk_TR0); --Clock inverted in Module
CLKB_TR1  <= (Clk_TR1); --Clock inverted in Module
CLKB_TR2  <= (Clk_TR2); --Clock inverted in Module
------------------------------------------------------
internal_gnd_TR0_s <= internal_gnd_TR0;
internal_gnd_TR1_s <= internal_gnd_TR1;
internal_gnd_TR2_s <= internal_gnd_TR2;
internal_vcc_TR0_s <= internal_vcc_TR0;
internal_vcc_TR1_s <= internal_vcc_TR1;
internal_vcc_TR2_s <= internal_vcc_TR2;
------------------------------------------------------------------------------------

CEN_bscntr_tmr_inst_TR0: CEN_bscntr_tmr 
port map (
     BOGUS_VCC => internal_vcc_TR0_s,
     CEN_TR0 => CEN_TR0, 
     CEN_TR1 => CEN_TR1, 
     CEN_TR2 => CEN_TR2, 
     CEN_bscntr_tmr => CEN_bscntr_tmr_TR0);

CEN_bscntr_tmr_inst_TR1: CEN_bscntr_tmr 
port map (
     BOGUS_VCC => internal_vcc_TR1_s,
     CEN_TR0 => CEN_TR0, 
     CEN_TR1 => CEN_TR1, 
     CEN_TR2 => CEN_TR2, 
     CEN_bscntr_tmr => CEN_bscntr_tmr_TR1);

CEN_bscntr_tmr_inst_TR2: CEN_bscntr_tmr 
port map (
     BOGUS_VCC => internal_vcc_TR2_s,
     CEN_TR0 => CEN_TR0, 
     CEN_TR1 => CEN_TR1, 
     CEN_TR2 => CEN_TR2, 
     CEN_bscntr_tmr => CEN_bscntr_tmr_TR2);

Comparator: Comparator_TMR
port map (

     ENB_TR0 => ENB_TR0_s,
     ENB_TR1 => ENB_TR1_s,
     ENB_TR2 => ENB_TR2_s,
     
     CEN_TR0 => CEN_TR0,
     CEN_TR1 => CEN_TR1,
     CEN_TR2 => CEN_TR2,

     ADDR_A_TR0 => Addr_TR0,
     ADDR_A_TR1 => Addr_TR1,
     ADDR_A_TR2 => Addr_TR2,

     WEN_A_TR0  => WE_TR0, 
     WEN_A_TR1  => WE_TR1, 
     WEN_A_TR2  => WE_TR2, 

     ADDR_B_TR0 => Addr_B_TR0,
     ADDR_B_TR1 => Addr_B_TR1,
     ADDR_B_TR2 => Addr_B_TR2,

     WEN_B_TR0 => WEN_B_TR0,
     WEN_B_TR1 => WEN_B_TR1,
     WEN_B_TR2 => WEN_B_TR2,

     ERR_B_TR0 => ERR_B_TR0,
     ERR_B_TR1 => ERR_B_TR1,
     ERR_B_TR2 => ERR_B_TR2
 );


scrub_counter: BSCNTR_XTMR
port map (
     internal_gnd_tr0 => internal_gnd_tr0_s,
     internal_gnd_tr1 => internal_gnd_tr1_s,
     internal_gnd_tr2 => internal_gnd_tr2_s,
     internal_vcc_tr0 => internal_vcc_tr0_s,
     internal_vcc_tr1 => internal_vcc_tr1_s,
     internal_vcc_tr2 => internal_vcc_tr2_s,
     CLK_TR0 => Clk_TR0, 
     CLK_TR1 => Clk_TR1, 
     CLK_TR2 => Clk_TR2, 
     SET_TR0 => SSR_TR0,
     SET_TR1 => SSR_TR1,
     SET_TR2 => SSR_TR2,
     RST_TR0 => SSR_TR0,
     RST_TR1 => SSR_TR1,
     RST_TR2 => SSR_TR2,
     CE_TR0 => CEN_bscntr_tmr_TR0,      
     CE_TR1 => CEN_bscntr_tmr_TR1,      
     CE_TR2 => CEN_bscntr_tmr_TR2,      
     ADDRESS_TR0 => Addr_B_TR0,
     ADDRESS_TR1 => Addr_B_TR1,
     ADDRESS_TR2 => Addr_B_TR2);

--This Generates the Triple Block RAMs and is inferred
-- use bram_scrubber_bram
-----------------------------------------------------------
bram_scrubber_bram_TR2: bram_scrubber_bram_bitw
     port map(
          CLKA  => CLK_TR2,
          ENA   => EN_TR2,
          SSRA    => SSR_TR2, --Add For Full HL Extraction
          WEA   => WE_TR2,
          DIA   => DI_TR2,
          ADDRA => ADDR_TR2,
          DOA   => DO_TR2_s,
          CLKB  => CLK_TR2,
          ENB   => ENB_TR2_s,
          SSRB  => SSR_TR2,   --Add For Full HL Extraction
          WEB   => WEN_B_TR2,
          DIB   => Din_B_TR2,
          ADDRB => Addr_B_TR2,
          DOB   => Dout_B_TR2
            );

bram_scrubber_bram_TR1: bram_scrubber_bram_bitw
     port map(
          CLKA  => CLK_TR1,
          ENA   => EN_TR1,
          SSRA    => SSR_TR1, --Add For Full HL Extraction
          WEA   => WE_TR1,
          DIA   => DI_TR1,
          ADDRA => ADDR_TR1,
          DOA   => DO_TR1_s,
          CLKB  => CLK_TR1,
          ENB   => ENB_TR1_s,
          SSRB  => SSR_TR1,   --Add For Full HL Extraction
          WEB   => WEN_B_TR1,
          DIB   => Din_B_TR1,
          ADDRB => Addr_B_TR1,
          DOB   => Dout_B_TR1
            );

bram_scrubber_bram_TR0: bram_scrubber_bram_bitw
     port map(
          CLKA  => CLK_TR0,
          ENA   => EN_TR0,
          SSRA   => SSR_TR0,  --Add For Full HL Extraction
          WEA   => WE_TR0,
          DIA   => DI_TR0,
          ADDRA => ADDR_TR0,
          DOA   => DO_TR0_s,
          CLKB  => CLK_TR0,
          ENB   => ENB_TR0_s,
          SSRB  => SSR_TR0,   --Add For Full HL Extraction
          WEB   => WEN_B_TR0,
          DIB   => Din_B_TR0,
          ADDRB => Addr_B_TR0,
          DOB   => Dout_B_TR0
            );

DO_TR0 <= DO_TR0_s;
DO_TR1 <= DO_TR1_s;
DO_TR2 <= DO_TR2_s;

-----------------------------------------------------------
bram_scrubber_data_voter_tr0: bram_scrubber_data_voter
     port map (
          TR0                 =>  Dout_B_TR0, --Prime
          TR1                 =>  Dout_B_TR1, --ReadA
          TR2                 =>  Dout_B_TR2, --ReadB
          Din_B               =>  Din_B_TR0,
          ERROR_B             =>  ERR_B_TR0
        );

bram_scrubber_data_voter_tr1: bram_scrubber_data_voter
     port map (
          TR0                 =>  Dout_B_TR1, --Prime
          TR1                 =>  Dout_B_TR2, --ReadA
          TR2                 =>  Dout_B_TR0, --ReadB
          Din_B               =>  Din_B_TR1,
          ERROR_B             =>  ERR_B_TR1
        );

bram_scrubber_data_voter_tr2: bram_scrubber_data_voter
     port map (
          TR0                 =>  Dout_B_TR2, --Prime
          TR1                 =>  Dout_B_TR0, --ReadA
          TR2                 =>  Dout_B_TR1, --ReadB
          Din_B               =>  Din_B_TR2,
          ERROR_B             =>  ERR_B_TR2
        );

--Optional Register output for Write Enable B

WEN_B_out : process(WEN_B_TR0, WEN_B_TR1, WEN_B_TR2)
begin
     if ( (WEN_B_TR0 = (WEN_B_TR0'range => '0')) ) then
               WEN_B_out_TR0_s <= '0';
          else
               WEN_B_out_TR0_s <= '1';
     end if;

     if ( (WEN_B_TR1 = (WEN_B_TR1'range => '0')) ) then
               WEN_B_out_TR1_s <= '0';
          else
               WEN_B_out_TR1_s <= '1';
     end if;

     if ( (WEN_B_TR2 = (WEN_B_TR2'range => '0')) ) then
               WEN_B_out_TR2_s <= '0';
          else
               WEN_B_out_TR2_s <= '1';
     end if;
end process;

------------------------------------
------Optional Testing Outputs------
---Pull Out for Testing-------------
--Check address conflict--

--WEN_B_out_TR0 <= WEN_B_out_TR0_s;
--WEN_B_out_TR1 <= WEN_B_out_TR1_s;
--WEN_B_out_TR2 <= WEN_B_out_TR2_s;

--Addr_B_out_TR0_s <= Addr_B_TR0(13 downto 10) & Addr_B_TR0(3 downto 0); --First and Last Nibble
--Addr_B_out_TR1_s <= Addr_B_TR1(13 downto 10) & Addr_B_TR1(3 downto 0); --First and Last Nibble
--Addr_B_out_TR2_s <= Addr_B_TR2(13 downto 10) & Addr_B_TR2(3 downto 0); --First and Last Nibble

--Addr_B_out_TR0 <= Addr_B_out_TR0_s;
--Addr_B_out_TR1 <= Addr_B_out_TR1_s;
--Addr_B_out_TR2 <= Addr_B_out_TR2_s;

------------------------------------

--------------------Chipscope Optional-----------------------------
-------------------------------------------------------------------
-- i_icon : icon
--   port map
--   (
--     control0   => control
--   );
---------------------------------------------------------------------
-- i_ila : ila
--   port map
--   (
--     control   => control,
--     clk       => cs_clk_TR0,
----     data      => trig0,
--     trig0     => trig0
--   );

-----------------------  Signals to observe --------------------------
--
--trig0(0)               <= internal_gnd_TR0_s;
--trig0(1)               <= internal_gnd_TR1_s;
--trig0(2)               <= internal_gnd_TR2_s;
--trig0(3)               <= internal_vcc_TR0_s;
--trig0(4)               <= internal_vcc_TR1_s;
--trig0(5)               <= internal_vcc_TR2_s;
--trig0(6)               <= CLK_TR0;
--trig0(7)               <= CLK_TR1;
--trig0(8)               <= CLK_TR2;
--trig0(9)            <= WEN_B_out_TR0_s;
--trig0(10)           <= WEN_B_out_TR1_s;
--trig0(11)           <= WEN_B_out_TR2_s;

--trig0(12)               <= CEN_TR0;
--trig0(13)               <= CEN_TR1;
--trig0(14)               <= CEN_TR2;

--trig0(15)               <= ENB_TR0_s;
--trig0(16)               <= ENB_TR1_s;
--trig0(17)               <= ENB_TR2_s;
----trig0(12-20) --reserved

--trig0(34 downto 21)      <= ADDR_TR0;
--trig0(48 downto 35)      <= ADDR_TR1;
--trig0(62 downto 49)      <= ADDR_TR2;

--trig0(78 downto 63)    <= Din_B_TR0;
--trig0(94 downto 79)    <= Din_B_TR1;
--trig0(110 downto 95)   <= Din_B_TR2;

--trig0(126 downto 111)  <= Dout_B_TR0;
--trig0(142 downto 127)  <= Dout_B_TR1;
--trig0(158 downto 143)  <= Dout_B_TR2;

--trig0(172 downto 159)  <= Addr_B_TR0;
--trig0(186 downto 173)  <= Addr_B_TR1;
--trig0(200 downto 187)  <= Addr_B_TR2;

--trig0(216 downto 201)  <= WEN_B_TR0;
--trig0(232 downto 217)  <= WEN_B_TR1;
--trig0(248 downto 233)  <= WEN_B_TR2;

-----------------End Chipscope Optional----------------------------


end architecture RTL;



